//
// timeslots.js
//
// 17 Aug, 2021 by FUJIWARA Teruyoshi <tf@dsl.gr.jp>
//
//
// Calculate 'open timeslots' using a list of planned timetables.
// slots are managed based on '5 minutes slots'.

// Usage:
// * See the sample file 'timeslots_sample.swf' for details.
//
// slots = new Timeslots();
// slots.remove("0:00", "8:00"); (and remove other planned slots)
// slots.print() (or you can use slots.save() for save the log file.)

//
// Setting
//
var fragment_min = 60; // drop small fragment less than this value (unit: minutes).

//
// Do not modify the following lines.
//
function Timeslot(start, end) {
  this.start = start;
  this.end = end;
}

function Timeslots() {
  this.array = new Array(new Timeslot(0, (60 / 5 * 24)));
}

Timeslots.prototype.setLabel = function (label) {
  this.label = label;
}

Timeslots.prototype.print = function () {
  var result = this.make_result();
  for (var i in result) {
    WScript.Echo(result[i]);
  }
}

Timeslots.prototype.save = function (filename) {
  var result = this.make_result();
  
  var fs = new ActiveXObject("Scripting.FileSystemObject");
  var file = fs.OpenTextFile(filename, 2, true);
  for (var i in result) {
    file.WriteLine(result[i]);
  }
  file.Close();
}

Timeslots.prototype.make_result = function () {
  var result = [];
  
  for (var i in this.array) {
    var a = this.array[i];

    if (a.end - a.start < (fragment_min / 5)) {
      continue;
    }
    var ds = Math.floor(a.start / 12);
    var rs = (a.start % 12) * 5;
    var de = Math.floor(a.end / 12);
    var re = (a.end % 12) * 5;
    var rs_str = ('00' + rs).slice(-2);
    var re_str = ('00' + re).slice(-2);
    result.push(ds + ":" + rs_str + "-" + de + ":" + re_str);
  }
  return result;
}

Timeslots.prototype.remove = function (start, end) {
  var arr_s = start.split(':');
  var arr_e = end.split(':');
  var index_s = (parseInt(arr_s[0], 10) * 12) + Math.floor(parseInt(arr_s[1], 10) / 5);
  var index_e = (parseInt(arr_e[0], 10) * 12) + Math.ceil(parseInt(arr_e[1], 10) / 5);

  return this.remove_index(index_s, index_e);
}

Timeslots.prototype.remove_index = function (start, end) {
  var new_array = [];
  for (var i in this.array) {
    var s = this.array[i].start;
    var e = this.array[i].end;
    //WScript.Echo(i + ":" + s + "-" + e + "/" + start + "-" + end);
    if (end <= s || start >= e) {
      // A or B: do nothing
      new_array.push(new Timeslot(s, e));
    } else if (start <= s && end < e) {
      // C: remove a part
      new_array.push(new Timeslot(end, e));
    } else if (start > s && end >= e) {
      // D: remove a part
      new_array.push(new Timeslot(s, start));
    } else if (start <= s && end >= e) {
      // E: remove all
    } else if (start > s && end < e) {
      // F: split the slot into two fragments
      new_array.push(new Timeslot(s, start));
      new_array.push(new Timeslot(end, e));
    } else {
      WScript.Echo("Never to be reached here!");
    }
  }
  this.array = new_array;
}

// Multi-days wrapper
function MDTimeslots (days) {
  this.label = [];
  this.datestr = [];
  this.table = [];
  var DayOfWeek = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
  for (var i = 0; i < days; i++) {
    var d = new Date();
    d.setDate(d.getDate() + i);
    var m = d.getMonth() + 1;
    var day = d.getDate();
    var dow = d.getDay();
    var s = m + "/" + day;
    this.label.push(s);
    this.datestr.push(s + "(" + DayOfWeek[dow] + ")");
    this.table.push(new Timeslots());
  }
}

MDTimeslots.prototype.save = function (filename) {
  var fs = new ActiveXObject("Scripting.FileSystemObject");
  var file = fs.OpenTextFile(filename, 2, true);

  for (var i in this.label) {
    file.WriteLine("-----------");
    file.WriteLine(this.datestr[i]);
    
    var result = this.table[i].make_result();
    for (var j in result) {
      file.WriteLine("  " + result[j]);
    }
  }
  file.Close();
}

MDTimeslots.prototype.print = function () {
  WScript.Echo("-----------");
  for (var i in this.label) {
    WScript.Echo(this.datestr[i]);
    this.table[i].print();
    WScript.Echo("-----------");
  }
}

MDTimeslots.prototype.remove = function (str, start, end) {
  var i = this.find(str);
  if (i < 0) {
    return;
  }
  this.table[i].remove(start, end);
}

MDTimeslots.prototype.find = function (str) {
  var i = 0;
  while (i < this.label.length) {
    if (this.label[i] == str) {
      return i;
    }
    i++;
  }
  return -1;
}
